const {dispatch, select} = wp.data;
const {PluginDocumentSettingPanel} = wp.editPost;
const {Component} = wp.element;
const {__} = wp.i18n;
const {SelectControl} = wp.components;

export default class Sidebar extends Component {

  constructor(props) {

    super(...arguments);

    //The state is only used to rerender the component with setState
    this.state = {
      statusValue: '',
    };

  }

  render() {

    const meta = select('core/editor').getEditedPostAttribute('meta');
    const statusMeta = meta['_helpful_pro_status'];

    const positiveValue = window.DAEXTHEFUP_PARAMETERS.statistics.positive_value;
    const negativeValue = window.DAEXTHEFUP_PARAMETERS.statistics.negative_value;
    const positivePercentageValue = window.DAEXTHEFUP_PARAMETERS.statistics.positive_percentage_value;
    const negativePercentageValue = window.DAEXTHEFUP_PARAMETERS.statistics.negative_percentage_value;

    return (
        <PluginDocumentSettingPanel
            name="daexthefup-options"
            title={__('Helpful', 'helpful-pro')}
        >
          <SelectControl
              label={__('Status', 'helpful-pro')}
              help={__(
                  'Select whether to enable or disable the form.',
                  'helpful-pro')}
              value={statusMeta}
              onChange={(status) => {

                dispatch('core/editor').editPost({
                  meta: {
                    '_helpful_pro_status': status,
                  },
                });

                //used to rerender the component
                this.setState({
                  statusValue: status,
                });

              }}
              options={[
                {label: 'Disabled', value: 0},
                {label: 'Enabled', value: 1},
              ]}
          />

          <div className="daexthefup-section-title">{__('Statistics', 'helpful-pro')}</div>
          <div className="daexthefup-stat daexthefup-positive-stat">
            <div className="daexthefup-stat-label">{__('Positive', 'helpful-pro')}</div>
            <div
                className="daexthefup-stat-value">{positiveValue} ({positivePercentageValue}%)
            </div>
          </div>
          <div className="daexthefup-stat daexthefup-negative-stat">
            <div className="daexthefup-stat-label">{__('Negative', 'helpful-pro')}</div>
            <div
                className="daexthefup-stat-value">{negativeValue} ({negativePercentageValue}%)
            </div>
          </div>

        </PluginDocumentSettingPanel>
    );
  }
}